import plotly.graph_objects as go

def job_stack(jobs, use_unit='cpu', fig_out='', plot_title='', query_bounds=True):
    """Create job stack figure based on a given DataFrame and specified use unit.

    Each job is a rectangle, where the height of the rectangle is based on
    the amount of resources that are being examined. Further, all three job
    states are displayed: queued, running, end time.

    Parameters
    -------
    jobs: DataFrame
        Job DataFrame typically generated by the ccmnt package.
    use_unit: str, optional
        Usage unit to examine. One of: {'cpu', 'cpu-eqv', 'gpu', 'gpu-eqv'}.
        Defaults to 'cpu'.
    fig_out: str, optional
        Writes the generated figure to file as specified.
        If empty, skips writing. Defaults to empty.
    plot_title: str, optional
        Title information passed to the figure object.
    query_bounds: bool, optional
        Draws red lines on the figure to represent where query is valid.
        Defaults to true.
    """

    if use_unit == 'cpu':
        jobs['use_unit'] = jobs['reqcpus']
    elif use_unit == 'cpu-eqv':
        jobs['mem_scale'] = jobs['mem'] / 4000.0
        jobs['use_unit'] = jobs[['mem_scale', 'reqcpus']].max(axis=1)
    elif use_unit == 'gpu':
        jobs['ngpus'] = jobs['r_tres'].str.extract(r'(\d+)(?!.*\d)').astype('float')
        jobs['use_unit'] = jobs['ngpus']
    else:
        print('gpu equiv to be supported soon')
        return

    cumu_sum_units = jobs['use_unit'].cumsum()

    res_count = 0
    x_queue = []
    x_run = []
    x_req = []

    # TODO: Fix this below section into frame calls
    y_cumu = []
    for _, row in jobs.iterrows():
        c_queue = [row['submit'], row['start'], row['start'], row['submit'], row['submit'], None]
        x_queue = x_queue + c_queue

        c_run = [row['start'], row['end'], row['end'], row['start'], row['start'], None]
        x_run = x_run + c_run

        c_req = [row['end'], row['start']+row['timelimit'],
                 row['start']+row['timelimit'], row['end'], row['end'], None]
        x_req = x_req + c_req

        c_cumres = [res_count, res_count, res_count+row['use_unit'],
                    res_count+row['use_unit'], res_count, None]
        y_cumu = y_cumu + c_cumres
        res_count = res_count+row['use_unit']

    fig = go.Figure()
    fig.add_trace(go.Scatter(
        x=x_queue,
        y=y_cumu,
        fill='toself',
        fillcolor='rgba(200,200,200,.5)',
        line_color='rgba(200,200,200,.3)',
        name='queued',
        hoveron='points+fills',
        text=jobs['jobid']
    ))
    fig.add_trace(go.Scatter(
        x=x_run,
        y=y_cumu,
        fill='toself',
        fillcolor='rgba(140,180,140,.9)',
        line_color='rgba(140,180,140,.1)',
        name='running'
    ))
    fig.add_trace(go.Scatter(
        x=x_req,
        y=y_cumu,
        fill='toself',
        fillcolor='rgba(120,120,180,.2)',
        line_color='rgba(120,120,180,.1)',
        name='requested'
    ))
    fig.add_trace(go.Scatter(
        x=jobs['submit'],
        y=cumu_sum_units-jobs['use_unit'],
        mode='markers',
        name='submit',
        marker_color='rgba(100,100,100,.3)',
        hovertext=jobs['jobid']
    ))
    fig.add_trace(go.Scatter(
        x=jobs['start'],
        y=cumu_sum_units-jobs['use_unit'],
        mode='markers',
        name='start',
        marker_color='rgba(20,120,20,.3)',
        hovertext=jobs['jobid']
    ))
    fig.add_trace(go.Scatter(
        x=jobs['end'],
        y=cumu_sum_units-jobs['use_unit'],
        mode='markers',
        name='end',
        marker_color='rgba(120,20,20,.3)',
        hovertext=jobs['jobid']
    ))
    fig.add_trace(go.Scatter(
        x=jobs['start']+jobs['timelimit'],
        y=cumu_sum_units-jobs['use_unit'],
        mode='markers',
        name='end',
        marker_color='rgba(100,100,140,.3)',
        hovertext=jobs['jobid']
    ))

    if query_bounds:
        Warning('Query bounds not yet implemented.')

    fig.update_layout(
        title_text="Job stack: "+plot_title,
        yaxis_title='Cumulative resources requested ('+str(use_unit)+')',
        xaxis_title='Date Time',
        showlegend=True)

    if fig_out != '':
        fig.write_html(fig_out)



from pathlib import Path
import viewclust as vc

def use_suite(clust_info, cores_queued, cores_running, folder, submit_run=[]):
    """Creates a folder of a given name and creates figures inside of it.

    Function is intended to be called in a loop over a list of accounts.
    Folder parameter should most often be the current account name.

    Parameters
    -------
    clust_info: DataFrame
        Frame which represents the cluster state at given time intervals. See jobUse.
    cores_queued: array_like of DataFrame
        Series displaying queued resources at a particular time. See jobUse.
    cores_running: array_like of DataFrame
        Series displaying running resources at a particular time. See jobUse.
    Folder: str
        Folder to place generated figures inside of.
        Typically an account name.
    submit_run: DataFrame, optional
        Draws a red line representing what would usage have looked like
        if jobs had started instantly. Allows for easier interpretation of
        the queued series. Defaults to not plotting.

    See Also
    -------
    jobUse: Generates the input frames for this function.
    summaryPage: Can build an html page based on all figures generated
    """

    # Handle folder creation
    safe_folder = folder
    if safe_folder[-1] != '/':
        safe_folder += '/'
    Path(safe_folder).mkdir(parents=True, exist_ok=True)

    # Add more to the suite as you like
    vc.cumu_plot(clust_info, cores_queued, cores_running, fig_out=safe_folder+'cumu_plot.html', submit_run=submit_run)
    vc.insta_plot(clust_info, cores_queued, cores_running, fig_out=safe_folder+'insta_plot.html', submit_run=submit_run)



import plotly.graph_objects as go

def delta_plot(account_list, dist_list, fig_out=''):
    """Takes a list of distance from target frames and generates the delta plot.

    Parameters
    -------
    account_list: array_like of DataFrame
        Strided to match dist_list for labels in the legend.
    dist_list: array_like of DataFrame
        Taken to be a list of distance from target series generated by the jobUse function.
    fig_out: str, optional
        Writes the generated figure to file as the given name.
        If empty, skips writing. Defaults to empty.

    See Also
    -------
    jobUse: Generates the input frame for this function.
    """

    fig = go.Figure()
    for i, frame in enumerate(dist_list):
        fig.add_trace(go.Scatter(x=frame.index, y=frame.divide(len(frame)),
                                 mode='lines',
                                 name=account_list[i],
                                 fillcolor='rgba(200, 128, 128, 1.0)'))
    if fig_out != '':
        fig.write_html(fig_out)



"""Top-level package for ViewClust-Vis."""

# Version output
from ._version import __version__

__author__ = """Tyler Collins"""
__email__ = 'tk11br@sharcnet.ca'

from .delta_plot import delta_plot
from .job_stack import job_stack
from .summary_page import summary_page
from .use_suite import use_suite
from .viol_plot import viol_plot



#!/usr/bin/env python

"""The setup script."""

import re
from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = ['pandas', 'numpy', 'plotly', 'viewclust']

setup_requirements = ['pandas', 'numpy', 'plotly', 'viewclust']

test_requirements = ['pandas', 'numpy', 'plotly', 'viewclust']

VERSIONFILE = "viewclust_vis/_version.py"
VERSTRLINE = open(VERSIONFILE, "rt").read()
VSRE = r"^__version__ = ['\"]([^'\"]*)['\"]"
MO = re.search(VSRE, VERSTRLINE, re.M)
if MO:
    VERSTR = MO.group(1)
else:
    raise RuntimeError("Unable to find version string in %s." % (VERSIONFILE,))

setup(
    author="Tyler Collins",
    author_email='tk11br@sharcnet.ca',
    python_requires='>=3.5',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="Extension to ViewClust containing dashboard submodules and more.",
    install_requires=requirements,
    license="MIT license",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='viewclust_vis',
    name='viewclust_vis',
    packages=find_packages(include=['viewclust_vis', 'viewclust_vis.*']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/Andesha/viewclust_vis',
    version=VERSTR,
    zip_safe=False,
)



from datetime import datetime
import plotly.graph_objects as go

def viol_plot(d_from, cores_queued, cores_running, target, d_to='', fig_out=''):
    """Violin distribution usage plot.

    Parameters
    -------
    d_from: date str
        Beginning of the query period, e.g. '2019-04-01T00:00:00'
    cores_queued: array_like of DataFrame
        Series displaying queued resources at a particular time. See jobUse.
    cores_running: array_like of DataFrame
        Series displaying running resources at a particular time. See jobUse.
    target: int
        Target information for a given usage. Can be user specific or system-wide.
    d_to: date str, optional
        End of the query period, e.g. '2020-01-01T00:00:00'. If empty, defaults to now.
    fig_out: str, optional
        Writes the generated figure to file as the given name.
        If empty, skips writing. Defaults to empty.

    See Also
    -------
    jobUse: Generates the input frames for this function.
    """

    # d_to boilerplate
    if d_to == '':
        now = datetime.now()
        d_to = now.strftime('%Y-%m-%dT%H:%M:%S')

    fig = go.Figure()
    fig.add_trace(go.Violin(y=cores_running.loc[d_from:d_to].divide(int(target)) * 100,
                            points='all', # show all points
                            jitter=0.05,  # add some jitter on points for better visibility
                            side='positive',
                            pointpos=-.1,
                            line_color='rgba(80,80,220, .8)',
                            name='Total CPUs running'))
    fig.add_trace(go.Violin(y=cores_queued.loc[d_from:d_to].divide(int(target)) * 100,
                            points='all', # show all points
                            jitter=0.05,  # add some jitter on points for better visibility
                            side='negative',
                            pointpos=.1,
                            line_color='rgba(220,80,80, .8)',
                            name='RAC CPUs running'))

    # update characteristics shared by all traces
    fig.update_traces(meanline_visible=True,
                      box_visible=True,
                      opacity=0.6,
                      scalemode='count') #scale violin plot area with total count

    fig.update_layout(
        title_text="CPU utilization: ",
        violingap=0, 
        violingroupgap=0, 
        violinmode='overlay',
        yaxis_title='Percent CPU capacity',
        xaxis_title='Time bin count',
        showlegend=False)

    if fig_out != '':
        fig.write_html(fig_out)



from datetime import datetime
import glob

def summary_page(folder_list, page_name):
    """Builds an html page containing links to all html files in a list of folders.

    There's probably a library to do this properly but I just wanted something low level for now.

    Parameters
    -------
    folder_list:Generates the input frames for this function.
        List of folders to check for html files.
    page_name: str
        Output html page name

    See Also
    -------
    useSuite: Generates multiple figures per account
    """

    out_page = """
    <!DOCTYPE html>
    <html>
    <body>

    <h1>ViewClust Summary Page: """ + str(datetime.now())+  """</h1> """

    for folder in folder_list:
        out_page += '<h2>' + folder + '</h2>'
        for plot in glob.glob(folder+'/*.html'):
            out_page += '<a href="'+plot+'">'+plot+'</a><br>'

    out_page += """
    </body>
    </html>
    """

    # Dump string as html page
    with open(page_name, 'w') as f_out:
        f_out.write(out_page)
